@@ -53,9 +53,6 @@ type BinaryInfo struct {
 
 	DebugInfoDirectories []string
 
-	// BuildID of this binary.
-	BuildID string
-
 	// Functions is a list of all DW_TAG_subprogram entries in debug_info, sorted by entry point
 	Functions []Function
 	// Sources is a list of all source files found in debug_line.
@@ -107,6 +104,8 @@ type BinaryInfo struct {
 	// dwrapUnwrapCache caches unwrapping of defer wrapper functions (dwrap)
 	dwrapUnwrapCache map[uint64]*Function
 
+	moduleDataCache []ModuleData
+
 	// Go 1.17 register ABI is enabled.
 	regabi bool
 
@@ -207,7 +206,6 @@ func FindFileLocation(p Process, filename string, lineno int) ([]uint64, error)
 	selectedPCs := []uint64{}
 
 	for fn, pcs := range pcByFunc {
-
 		// 3. for each concrete function split instruction between the inlined functions it contains
 
 		if strings.Contains(fn.Name, "·dwrap·") || fn.trampoline {
@@ -467,7 +465,7 @@ type compileUnit struct {
 	entry     *dwarf.Entry        // debug_info entry describing this compile unit
 	isgo      bool                // true if this is the go compile unit
 	lineInfo  *line.DebugLineInfo // debug_line segment associated with this compile unit
-	optimized bool                // this compile unit is optimized
+	optimized optimizedFlags      // this compile unit is optimized
 	producer  string              // producer attribute
 
 	offset dwarf.Offset // offset of the entry describing the compile unit
@@ -475,6 +473,13 @@ type compileUnit struct {
 	image *Image // parent image of this compilation unit.
 }
 
+type optimizedFlags uint8
+
+const (
+	optimizedInlined optimizedFlags = 1 << iota
+	optimizedOptimized
+)
+
 type fileLine struct {
 	file string
 	line int
@@ -502,7 +507,24 @@ type Function struct {
 	trampoline bool // DW_AT_trampoline attribute set to true
 
 	// InlinedCalls lists all inlined calls to this function
-	InlinedCalls []InlinedCall
+	InlinedCalls         []InlinedCall
+	rangeParentNameCache int // see rangeParentName
+	// extraCache contains information about this function that is only needed for
+	// some operations and is expensive to compute or store for every function.
+	extraCache *functionExtra
+}
+
+type functionExtra struct {
+	// closureStructType is the cached struct type for closures for this function
+	closureStructType *godwarf.StructType
+
+	// rangeParent is set when this function is a range-over-func body closure
+	// and points to the function that the closure was generated from.
+	rangeParent *Function
+	// rangeBodies is the list of range-over-func body closures for this
+	// function. Only one between rangeParent and rangeBodies should be set at
+	// any given time.
+	rangeBodies []*Function
 }
 
 // instRange returns the indexes in fn.Name of the type parameter
@@ -592,7 +614,7 @@ func (fn *Function) NameWithoutTypeParams() string {
 
 // Optimized returns true if the function was optimized by the compiler.
 func (fn *Function) Optimized() bool {
-	return fn.cu.optimized
+	return fn.cu.optimized != 0
 }
 
 // PrologueEndPC returns the PC just after the function prologue
@@ -639,6 +661,106 @@ func (fn *Function) privateRuntime() bool {
 	return len(name) > n && name[:n] == "runtime." && !('A' <= name[n] && name[n] <= 'Z')
 }
 
+func rangeParentName(fnname string) int {
+	const rangeSuffix = "-range"
+	ridx := strings.Index(fnname, rangeSuffix)
+	if ridx <= 0 {
+		return -1
+	}
+	ok := true
+	for i := ridx + len(rangeSuffix); i < len(fnname); i++ {
+		if fnname[i] < '0' || fnname[i] > '9' {
+			ok = false
+			break
+		}
+	}
+	if !ok {
+		return -1
+	}
+	return ridx
+}
+
+// rangeParentName, if this function is a range-over-func body closure
+// returns the name of the parent function, otherwise returns ""
+func (fn *Function) rangeParentName() string {
+	if fn.rangeParentNameCache == 0 {
+		ridx := rangeParentName(fn.Name)
+		fn.rangeParentNameCache = ridx
+	}
+	if fn.rangeParentNameCache < 0 {
+		return ""
+	}
+	return fn.Name[:fn.rangeParentNameCache]
+}
+
+// extra loads information about fn that is expensive to compute and we
+// only need for a minority of the functions.
+func (fn *Function) extra(bi *BinaryInfo) *functionExtra {
+	if fn.extraCache != nil {
+		return fn.extraCache
+	}
+
+	if fn.cu.image.Stripped() {
+		fn.extraCache = &functionExtra{}
+		return fn.extraCache
+	}
+
+	fn.extraCache = &functionExtra{}
+
+	// Calculate closureStructType
+	{
+		dwarfTree, err := fn.cu.image.getDwarfTree(fn.offset)
+		if err != nil {
+			return nil
+		}
+		st := &godwarf.StructType{
+			Kind: "struct",
+		}
+		vars := reader.Variables(dwarfTree, 0, 0, reader.VariablesNoDeclLineCheck|reader.VariablesSkipInlinedSubroutines)
+		for _, v := range vars {
+			off, ok := v.Val(godwarf.AttrGoClosureOffset).(int64)
+			if ok {
+				n, _ := v.Val(dwarf.AttrName).(string)
+				typ, err := v.Type(fn.cu.image.dwarf, fn.cu.image.index, fn.cu.image.typeCache)
+				if err == nil {
+					sz := typ.Common().ByteSize
+					st.Field = append(st.Field, &godwarf.StructField{
+						Name:       n,
+						Type:       typ,
+						ByteOffset: off,
+						ByteSize:   sz,
+						BitOffset:  off * 8,
+						BitSize:    sz * 8,
+					})
+				}
+			}
+		}
+
+		if len(st.Field) > 0 {
+			lf := st.Field[len(st.Field)-1]
+			st.ByteSize = lf.ByteOffset + lf.Type.Common().ByteSize
+		}
+		fn.extraCache.closureStructType = st
+	}
+
+	// Find rangeParent for this function (if it is a range-over-func body closure)
+	if rangeParentName := fn.rangeParentName(); rangeParentName != "" {
+		fn.extraCache.rangeParent = bi.lookupOneFunc(rangeParentName)
+	}
+
+	// Find range-over-func bodies of this function
+	if fn.extraCache.rangeParent == nil {
+		for i := range bi.Functions {
+			fn2 := &bi.Functions[i]
+			if strings.HasPrefix(fn2.Name, fn.Name) && fn2.rangeParentName() == fn.Name {
+				fn.extraCache.rangeBodies = append(fn.extraCache.rangeBodies, fn2)
+			}
+		}
+	}
+
+	return fn.extraCache
+}
+
 type constantsMap map[dwarfRef]*constantType
 
 type constantType struct {
@@ -836,6 +958,7 @@ func (bi *BinaryInfo) PCToImage(pc uint64) *Image {
 type Image struct {
 	Path       string
 	StaticBase uint64
+	BuildID    string
 	addr       uint64
 
 	index int // index of this object in BinaryInfo.SharedObjects
@@ -912,7 +1035,7 @@ func (bi *BinaryInfo) AddImage(path string, addr uint64) error {
 }
 
 // moduleDataToImage finds the image corresponding to the given module data object.
-func (bi *BinaryInfo) moduleDataToImage(md *moduleData) *Image {
+func (bi *BinaryInfo) moduleDataToImage(md *ModuleData) *Image {
 	fn := bi.PCToFunc(md.text)
 	if fn != nil {
 		return bi.funcToImage(fn)
@@ -931,7 +1054,7 @@ func (bi *BinaryInfo) moduleDataToImage(md *moduleData) *Image {
 }
 
 // imageToModuleData finds the module data in mds corresponding to the given image.
-func (bi *BinaryInfo) imageToModuleData(image *Image, mds []moduleData) *moduleData {
+func (bi *BinaryInfo) imageToModuleData(image *Image, mds []ModuleData) *ModuleData {
 	for _, md := range mds {
 		im2 := bi.moduleDataToImage(&md)
 		if im2 != nil && im2.index == image.index {
@@ -1321,10 +1444,10 @@ func (bi *BinaryInfo) openSeparateDebugInfo(image *Image, exe *elf.File, debugIn
 		}
 	}
 
-	if debugFilePath == "" && len(bi.BuildID) > 2 {
+	if debugFilePath == "" && len(image.BuildID) > 2 {
 		// Build ID method: look for a file named .build-id/nn/nnnnnnnn.debug in
 		// every debug info directory.
-		find(nil, fmt.Sprintf(".build-id/%s/%s.debug", bi.BuildID[:2], bi.BuildID[2:]))
+		find(nil, fmt.Sprintf(".build-id/%s/%s.debug", image.BuildID[:2], image.BuildID[2:]))
 	}
 
 	if debugFilePath == "" {
@@ -1358,16 +1481,15 @@ func (bi *BinaryInfo) openSeparateDebugInfo(image *Image, exe *elf.File, debugIn
 					bi.logger.Errorf("gnu_debuglink CRC check failed for %s (want %x got %x)", debugFilePath, crc, computedCRC)
 					debugFilePath = ""
 				}
-
 			}
 		}
 	}
 
-	if debugFilePath == "" && len(bi.BuildID) > 2 {
+	if debugFilePath == "" && len(image.BuildID) > 2 {
 		// Previous versions of delve looked for the build id in every debug info
 		// directory that contained the build-id substring. This behavior deviates
 		// from the ones specified by GDB but we keep it for backwards compatibility.
-		find(func(dir string) bool { return strings.Contains(dir, "build-id") }, fmt.Sprintf("%s/%s.debug", bi.BuildID[:2], bi.BuildID[2:]))
+		find(func(dir string) bool { return strings.Contains(dir, "build-id") }, fmt.Sprintf("%s/%s.debug", image.BuildID[:2], image.BuildID[2:]))
 	}
 
 	if debugFilePath == "" {
@@ -1382,7 +1504,7 @@ func (bi *BinaryInfo) openSeparateDebugInfo(image *Image, exe *elf.File, debugIn
 	// has debuginfod so that we can use that in order to find any relevant debug information.
 	if debugFilePath == "" {
 		var err error
-		debugFilePath, err = debuginfod.GetDebuginfo(bi.BuildID)
+		debugFilePath, err = debuginfod.GetDebuginfo(image.BuildID)
 		if err != nil {
 			return nil, nil, ErrNoDebugInfoFound
 		}
@@ -1455,63 +1577,10 @@ func loadBinaryInfoElf(bi *BinaryInfo, image *Image, path string, addr uint64, w
 			if len(bi.Images) <= 1 {
 				fmt.Fprintln(os.Stderr, "Warning: no debug info found, some functionality will be missing such as stack traces and variable evaluation.")
 			}
-			cu := &compileUnit{}
-			cu.image = image
-			symTable, symTabAddr, err := readPcLnTableElf(elfFile, path)
+			err := loadBinaryInfoGoRuntimeElf(bi, image, path, elfFile)
 			if err != nil {
 				return fmt.Errorf("could not read debug info (%v) and could not read go symbol table (%v)", dwerr, err)
 			}
-			image.symTable = symTable
-			noPtrSectionData, err := elfFile.Section(".noptrdata").Data()
-			if err != nil {
-				return err
-			}
-			md, err := parseModuleData(noPtrSectionData, symTabAddr)
-			if err != nil {
-				return err
-			}
-			roDataAddr := elfFile.Section(".rodata").Addr
-			goFuncVal, err := findGoFuncVal(md, roDataAddr, bi.Arch.ptrSize)
-			if err != nil {
-				return err
-			}
-			prog := gosym.ProgContaining(elfFile, goFuncVal)
-			inlFuncs := make(map[string]*Function)
-			for _, f := range image.symTable.Funcs {
-				fnEntry := f.Entry + image.StaticBase
-				if prog != nil {
-					inlCalls, err := image.symTable.GetInlineTree(&f, goFuncVal, prog.Vaddr, prog.ReaderAt)
-					if err != nil {
-						return err
-					}
-					for _, inlfn := range inlCalls {
-						newInlinedCall := InlinedCall{cu: cu, LowPC: fnEntry + uint64(inlfn.ParentPC)}
-						if fn, ok := inlFuncs[inlfn.Name]; ok {
-							fn.InlinedCalls = append(fn.InlinedCalls, newInlinedCall)
-							continue
-						}
-						inlFuncs[inlfn.Name] = &Function{
-							Name:  inlfn.Name,
-							Entry: 0, End: 0,
-							cu: cu,
-							InlinedCalls: []InlinedCall{
-								newInlinedCall,
-							},
-						}
-					}
-				}
-				fn := Function{Name: f.Name, Entry: fnEntry, End: f.End + image.StaticBase, cu: cu}
-				bi.Functions = append(bi.Functions, fn)
-			}
-			for i := range inlFuncs {
-				bi.Functions = append(bi.Functions, *inlFuncs[i])
-			}
-			sort.Sort(functionsDebugInfoByEntry(bi.Functions))
-			for f := range image.symTable.Files {
-				bi.Sources = append(bi.Sources, f)
-			}
-			sort.Strings(bi.Sources)
-			bi.Sources = uniq(bi.Sources)
 			return nil
 		}
 		image.sepDebugCloser = sepFile
@@ -1646,7 +1715,7 @@ func (bi *BinaryInfo) loadBuildID(image *Image, file *elf.File) {
 		bi.logger.Warnf("can't read build-id desc: %v", err)
 		return
 	}
-	bi.BuildID = hex.EncodeToString(descBinary)
+	image.BuildID = hex.EncodeToString(descBinary)
 }
 
 func (bi *BinaryInfo) getDebugLink(exe *elf.File) (debugLink string, crc uint32) {
@@ -1846,18 +1915,18 @@ func (bi *BinaryInfo) setGStructOffsetPE(entryPoint uint64, peFile *pe.File) {
 		producer := bi.Producer()
 		if producer != "" && goversion.ProducerAfterOrEqual(producer, 1, 20) {
 			// Use runtime.tls_g as pointer to offset from GS to G struct:
-			// https://golang.org/src/runtime/sys_windows_amd64.s
+			// https://go.dev/src/runtime/sys_windows_amd64.s
 			bi.gStructOffset = readtls_g()
 			bi.gStructOffsetIsPtr = true
 		} else {
 			// Use ArbitraryUserPointer (0x28) as pointer to pointer
 			// to G struct per:
-			// https://golang.org/src/runtime/cgo/gcc_windows_amd64.c
+			// https://go.dev/src/runtime/cgo/gcc_windows_amd64.c
 			bi.gStructOffset = 0x28
 		}
 	case _IMAGE_FILE_MACHINE_ARM64:
 		// Use runtime.tls_g as pointer to offset from R18 to G struct:
-		// https://golang.org/src/runtime/sys_windows_arm64.s
+		// https://go.dev/src/runtime/sys_windows_arm64.s
 		bi.gStructOffset = readtls_g()
 		bi.gStructOffsetIsPtr = true
 	}
@@ -1888,7 +1957,6 @@ func (bi *BinaryInfo) parseDebugFramePE(image *Image, exe *pe.File, debugInfoByt
 // loadBinaryInfoMacho specifically loads information from a Mach-O binary.
 func loadBinaryInfoMacho(bi *BinaryInfo, image *Image, path string, entryPoint uint64, wg *sync.WaitGroup) error {
 	exe, err := macho.Open(path)
-
 	if err != nil {
 		return err
 	}
@@ -1912,68 +1980,16 @@ func loadBinaryInfoMacho(bi *BinaryInfo, image *Image, path string, entryPoint u
 		return &ErrUnsupportedArch{os: "darwin", cpuArch: exe.Cpu}
 	}
 	var dwerr error
+	macOSShortSectionNamesWorkaround(exe)
 	image.dwarf, dwerr = exe.DWARF()
 	if dwerr != nil {
 		if len(bi.Images) <= 1 {
 			fmt.Fprintln(os.Stderr, "Warning: no debug info found, some functionality will be missing such as stack traces and variable evaluation.")
 		}
-		symTable, symTabAddr, err := readPcLnTableMacho(exe, path)
+		err := loadBinaryInfoGoRuntimeMacho(bi, image, path, exe)
 		if err != nil {
 			return fmt.Errorf("could not read debug info (%v) and could not read go symbol table (%v)", dwerr, err)
 		}
-		image.symTable = symTable
-		cu := &compileUnit{}
-		cu.image = image
-		noPtrSectionData, err := exe.Section("__noptrdata").Data()
-		if err != nil {
-			return err
-		}
-		md, err := parseModuleData(noPtrSectionData, symTabAddr)
-		if err != nil {
-			return err
-		}
-		roDataAddr := exe.Section("__rodata").Addr
-		goFuncVal, err := findGoFuncVal(md, roDataAddr, bi.Arch.ptrSize)
-		if err != nil {
-			return err
-		}
-		seg := gosym.SegmentContaining(exe, goFuncVal)
-		inlFuncs := make(map[string]*Function)
-		for _, f := range image.symTable.Funcs {
-			fnEntry := f.Entry + image.StaticBase
-			if seg != nil {
-				inlCalls, err := image.symTable.GetInlineTree(&f, goFuncVal, seg.Addr, seg.ReaderAt)
-				if err != nil {
-					return err
-				}
-				for _, inlfn := range inlCalls {
-					newInlinedCall := InlinedCall{cu: cu, LowPC: fnEntry + uint64(inlfn.ParentPC)}
-					if fn, ok := inlFuncs[inlfn.Name]; ok {
-						fn.InlinedCalls = append(fn.InlinedCalls, newInlinedCall)
-						continue
-					}
-					inlFuncs[inlfn.Name] = &Function{
-						Name:  inlfn.Name,
-						Entry: 0, End: 0,
-						cu: cu,
-						InlinedCalls: []InlinedCall{
-							newInlinedCall,
-						},
-					}
-				}
-			}
-			fn := Function{Name: f.Name, Entry: fnEntry, End: f.End + image.StaticBase, cu: cu}
-			bi.Functions = append(bi.Functions, fn)
-		}
-		for i := range inlFuncs {
-			bi.Functions = append(bi.Functions, *inlFuncs[i])
-		}
-		sort.Sort(functionsDebugInfoByEntry(bi.Functions))
-		for f := range image.symTable.Files {
-			bi.Sources = append(bi.Sources, f)
-		}
-		sort.Strings(bi.Sources)
-		bi.Sources = uniq(bi.Sources)
 		return nil
 	}
 	debugInfoBytes, err := godwarf.GetDebugSectionMacho(exe, "info")
@@ -2127,6 +2143,164 @@ func (bi *BinaryInfo) macOSDebugFrameBugWorkaround() {
 	}
 }
 
+// macOSShortSectionNamesWorkaround works around a bug in Go 1.23 (and
+// earlier).
+// Section names in Macho-O executables are limited to 16 characters, which
+// means that some DWARF sections with long names will be truncated. Go 1.23
+// and prior do not take into account this making the DWARF info sometimes
+// unreadable.
+// This bug only manifests on macOS 15 because the C toolchain of prior
+// versions of the operating system did not emit problematic DWARF sections.
+// See also https://github.com/go-delve/delve/issues/3797
+func macOSShortSectionNamesWorkaround(exe *macho.File) {
+	for _, sec := range exe.Sections {
+		if sec == nil {
+			continue
+		}
+		for _, longname := range []string{
+			"__debug_str_offsets",
+			"__zdebug_line_str",
+			"__zdebug_loclists",
+			"__zdebug_pubnames",
+			"__zdebug_pubtypes",
+			"__zdebug_rnglists",
+			"__zdebug_str_offsets",
+		} {
+			if sec.Name == longname[:16] {
+				logflags.DebuggerLogger().Debugf("expanding section name %q to %q", sec.Name, longname)
+				sec.Name = longname
+				break
+			}
+		}
+	}
+}
+
+// GO RUNTIME INFO ////////////////////////////////////////////////////////////
+
+// loadBinaryInfoGoRuntimeElf loads information from the Go runtime sections
+// of an ELF binary, it is only called when debug info has been stripped.
+func loadBinaryInfoGoRuntimeElf(bi *BinaryInfo, image *Image, path string, elfFile *elf.File) (err error) {
+	// This is a best-effort procedure, it can go wrong in unexpected ways, so
+	// recover all panics.
+	defer func() {
+		ierr := recover()
+		if ierr != nil {
+			err = fmt.Errorf("error loading binary info from Go runtime: %v", ierr)
+		}
+	}()
+
+	cu := &compileUnit{}
+	cu.image = image
+	symTable, symTabAddr, err := readPcLnTableElf(elfFile, path)
+	if err != nil {
+		return err
+	}
+	image.symTable = symTable
+	noPtrSectionData, err := elfFile.Section(".noptrdata").Data()
+	if err != nil {
+		return err
+	}
+	md, err := parseModuleData(noPtrSectionData, symTabAddr)
+	if err != nil {
+		return err
+	}
+	roDataAddr := elfFile.Section(".rodata").Addr
+	goFuncVal, err := findGoFuncVal(md, roDataAddr, bi.Arch.ptrSize)
+	if err != nil {
+		return err
+	}
+	prog := gosym.ProgContaining(elfFile, goFuncVal)
+	var progAddr uint64
+	var progReaderAt io.ReaderAt
+	if prog != nil {
+		progAddr = prog.Vaddr
+		progReaderAt = prog.ReaderAt
+	}
+	return loadBinaryInfoGoRuntimeCommon(bi, image, cu, goFuncVal, progAddr, progReaderAt)
+}
+
+// loadBinaryInfoGoRuntimeMacho loads information from the Go runtime sections
+// of an Macho-o binary, it is only called when debug info has been stripped.
+func loadBinaryInfoGoRuntimeMacho(bi *BinaryInfo, image *Image, path string, exe *macho.File) (err error) {
+	// This is a best-effort procedure, it can go wrong in unexpected ways, so
+	// recover all panics.
+	defer func() {
+		ierr := recover()
+		if ierr != nil {
+			err = fmt.Errorf("error loading binary info from Go runtime: %v", ierr)
+		}
+	}()
+
+	cu := &compileUnit{}
+	cu.image = image
+	symTable, symTabAddr, err := readPcLnTableMacho(exe, path)
+	if err != nil {
+		return err
+	}
+	image.symTable = symTable
+	noPtrSectionData, err := exe.Section("__noptrdata").Data()
+	if err != nil {
+		return err
+	}
+	md, err := parseModuleData(noPtrSectionData, symTabAddr)
+	if err != nil {
+		return err
+	}
+	roDataAddr := exe.Section("__rodata").Addr
+	goFuncVal, err := findGoFuncVal(md, roDataAddr, bi.Arch.ptrSize)
+	if err != nil {
+		return err
+	}
+	seg := gosym.SegmentContaining(exe, goFuncVal)
+	var segAddr uint64
+	var segReaderAt io.ReaderAt
+	if seg != nil {
+		segAddr = seg.Addr
+		segReaderAt = seg.ReaderAt
+	}
+	return loadBinaryInfoGoRuntimeCommon(bi, image, cu, goFuncVal, segAddr, segReaderAt)
+}
+
+func loadBinaryInfoGoRuntimeCommon(bi *BinaryInfo, image *Image, cu *compileUnit, goFuncVal uint64, goFuncSegAddr uint64, goFuncReader io.ReaderAt) error {
+	inlFuncs := make(map[string]*Function)
+	for _, f := range image.symTable.Funcs {
+		fnEntry := f.Entry + image.StaticBase
+		if goFuncReader != nil {
+			inlCalls, err := image.symTable.GetInlineTree(&f, goFuncVal, goFuncSegAddr, goFuncReader)
+			if err != nil {
+				return err
+			}
+			for _, inlfn := range inlCalls {
+				newInlinedCall := InlinedCall{cu: cu, LowPC: fnEntry + uint64(inlfn.ParentPC)}
+				if fn, ok := inlFuncs[inlfn.Name]; ok {
+					fn.InlinedCalls = append(fn.InlinedCalls, newInlinedCall)
+					continue
+				}
+				inlFuncs[inlfn.Name] = &Function{
+					Name:  inlfn.Name,
+					Entry: 0, End: 0,
+					cu: cu,
+					InlinedCalls: []InlinedCall{
+						newInlinedCall,
+					},
+				}
+			}
+		}
+		fn := Function{Name: f.Name, Entry: fnEntry, End: f.End + image.StaticBase, cu: cu}
+		bi.Functions = append(bi.Functions, fn)
+	}
+	for i := range inlFuncs {
+		bi.Functions = append(bi.Functions, *inlFuncs[i])
+	}
+	sort.Sort(functionsDebugInfoByEntry(bi.Functions))
+	for f := range image.symTable.Files {
+		bi.Sources = append(bi.Sources, f)
+	}
+	sort.Strings(bi.Sources)
+	bi.Sources = uniq(bi.Sources)
+	return nil
+}
+
 // Do not call this function directly it isn't able to deal correctly with package paths
 func (bi *BinaryInfo) findType(name string) (godwarf.Type, error) {
 	name = strings.ReplaceAll(name, "interface{", "interface {")
@@ -2307,9 +2481,18 @@ func (bi *BinaryInfo) loadDebugInfoMaps(image *Image, debugInfoBytes, debugLineB
 			if cu.isgo && cu.producer != "" {
 				semicolon := strings.Index(cu.producer, ";")
 				if semicolon < 0 {
-					cu.optimized = goversion.ProducerAfterOrEqual(cu.producer, 1, 10)
+					cu.optimized = 0
+					if goversion.ProducerAfterOrEqual(cu.producer, 1, 10) {
+						cu.optimized = optimizedInlined | optimizedOptimized
+					}
 				} else {
-					cu.optimized = !strings.Contains(cu.producer[semicolon:], "-N") || !strings.Contains(cu.producer[semicolon:], "-l")
+					cu.optimized = optimizedInlined | optimizedOptimized
+					if strings.Contains(cu.producer[semicolon:], "-N") {
+						cu.optimized &^= optimizedOptimized
+					}
+					if strings.Contains(cu.producer[semicolon:], "-l") {
+						cu.optimized &^= optimizedInlined
+					}
 					const regabi = " regabi"
 					if i := strings.Index(cu.producer[semicolon:], regabi); i > 0 {
 						i += semicolon
